/****************************************************************************
* File Name: BLE_HTSS.c
*
* Version: 1.0
*
* Description:
* This file implements functions related to BLE.
*
* Note:
* 
* Owner:
* PMAD
*
******************************************************************************
* Copyright (2014), Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/


/*****************************************************************************
* Included headers
*****************************************************************************/
#include "stdbool.h"
#include "BLE_HTSS.h"
#include "CommonFunctions.h"
#include "WatchdogTimer.h"

/*****************************************************************************
* Macros and constants
*****************************************************************************/
#define             REFRESH_INTERVAL                    (1000u)
#define             HTSS_TEMPERATURE_MEASURE_SIZE       (5u)

/* htssInterval holds the Health Thermometer Service measurement interval */
uint32 htssInterval = DEFAULT_MEASUREMENT_INTERVAL;

/* Indicates the status of Indication */
static bool htssIndication = false;

/* Connection handle used for transmitting information over BLE */
CYBLE_CONN_HANDLE_T  connectionHandle;	

/*****************************************************************************
* Function Name: GenericEventHandler()
******************************************************************************
* Summary:
* Handles the generic events generated by stack and take necessary actions 
* when event is received.
*
* Parameters:
*  eventCode - the event code
*  *event_params - the event parameters
*
* Return:
*  None.
*
* Side Effects:
* None
*
* Note:
*
*****************************************************************************/
void GenericEventHandler(uint32 event, void *eventParam)
{
  switch(event)
  {
    /* This event is received when component is Started */
    case CYBLE_EVT_STACK_ON: 					
      /* Stop watchdog to reduce power consumption during advertising */
      WatchdogTimer_Stop();
      /* Start Advertisement and enter Discoverable mode*/
      CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);			
    break;
          
    /* This event is received when device is disconnected or advetising times out*/
    case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:		
    case CYBLE_EVT_TIMEOUT:                     
      /* Sets the ENABLE_HIBERNATE flag to put system in Hibernate mode */
      SystemFlag |= ENABLE_HIBERNATE;
    break;
    
    /* This event is received when connection is established */
    case CYBLE_EVT_GATT_CONNECT_IND:
        /* Start watchdog timer with 1s refresh interval */
        /* Note: For this application, wakeup should be 1s because htssInterval
        *  resolution is configured as 1s */
        WatchdogTimer_Start(REFRESH_INTERVAL);
        /* Retrieve BLE connection handle */
        connectionHandle = *(CYBLE_CONN_HANDLE_T *) eventParam;
    break;
        
    default:
      /* Error handling */
    break;
  } 
}

/*****************************************************************************
* Function Name: HtssEventHandler()
******************************************************************************
* Summary:
* Handles the event generated for the health thermometer service.
*
* Parameters:
*  eventCode - the event code
*  *event_params - the event parameters
*
* Return:
*  None.
*
* Side Effects:
* None
*
* Note:
*
*****************************************************************************/
void HtssEventHandler(uint32 event, void* eventParam)
{
  CYBLE_HTS_CHAR_VALUE_T *interval;
  switch(event)
  {
    /* This event is received when indication are enabled by the central */
    case CYBLE_EVT_HTSS_INDICATION_ENABLED:
      /* Set the htssIndication flag */
      htssIndication = true;
    break;

    /* This event is received when indication are disabled by the central */
    case CYBLE_EVT_HTSS_INDICATION_DISABLED:
      /* Reset the htssIndiciation flag */
      htssIndication = false;
    break;

    /* This event is received when measurement interval is updated by
    *  the central */
    case CYBLE_EVT_HTSS_CHAR_WRITE:
      /* Retrive interval value */
      interval = ((CYBLE_HTS_CHAR_VALUE_T *)eventParam);
      htssInterval = interval->value->val[1];
      /* Update htssInterval with the updated value */
      htssInterval = (htssInterval << 8) | interval->value->val[0];
    break;

    default:
      /* Error handling */
    break;
  }
}

/*****************************************************************************
* Function Name: EnableBLE()
******************************************************************************
* Summary:
* Starts the BLE component and register the call back functions for 
* common events, heart rate service specific events and watchdog interrupt.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Side Effects:
* None
*
* Note:
*
*****************************************************************************/
void EnableBLE (void)
{
    /* Enable BLE block and register the callback function for common events */
    CyBle_Start(GenericEventHandler);

    /* Regsiters the callback function for heart rate service specific events */
    CyBle_HtsRegisterAttrCallback(HtssEventHandler);
}

/*****************************************************************************
* Function Name: ProcessBLE()
******************************************************************************
* Summary:
* If health thermometer indiications are enabled and device is connected then this
* functions transmits the temeprature information at htssInterval.
*
* Parameters:
*  uint32 - Measured temperature value.
*
* Return:
*  None.
*
* Side Effects:
* None
*
* Note:
*
*****************************************************************************/
void ProcessBLE (uint32 temperature)
{
    /* Temporary array to hold Health Thermometer characteristic information */
    uint8 valueArray[HTSS_TEMPERATURE_MEASURE_SIZE];
    
    /* Check if indications are enabled and measurement interval has elapsed */
    if((true == htssIndication) && (SystemFlag & MEASUREMENT_INTERVAL))
    {
        /* Clear measurement interval flag */
        SystemFlag &= ~MEASUREMENT_INTERVAL;
        
        /* Read Health Thermometer Characteristic from GATT DB */
        if(CYBLE_ERROR_OK == CyBle_HtssGetCharacteristicValue(CYBLE_HTS_TEMP_MEASURE, HTSS_TEMPERATURE_MEASURE_SIZE, valueArray))
        {
            /* Update temperature in the characteristic */
            valueArray[1] = LO8(LO16(temperature));
            valueArray[2] = HI8(LO16(temperature));
            valueArray[3] = LO8(HI16(temperature));
            valueArray[4] = HI8(HI16(temperature));
            /* Send indication to the central */
            CyBle_HtssSendIndication(connectionHandle, CYBLE_HTS_TEMP_MEASURE, HTSS_TEMPERATURE_MEASURE_SIZE, valueArray);
        }
    }
}

/* [] END OF FILE */
